@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2013-2014 eNovance SAS <licensing@enovance.com>
+# Copyright (C) 2013-2015 eNovance SAS <licensing@enovance.com>
 #
 # Author: Frederic Lepied <frederic.lepied@enovance.com>
 #
@@ -15,11 +15,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-'''Main entry point for hardware and system detection routines in eDeploy.'''
+"""Main entry point for hardware and system detection routines in eDeploy."""
 
 
+from __future__ import print_function
+import argparse
 import fcntl
-import getopt
 import json
 import os
 import pprint
@@ -29,10 +30,16 @@ import struct
 from subprocess import PIPE
 from subprocess import Popen
 import sys
+import uuid
 import xml.etree.ElementTree as ET
 
 from netaddr import IPNetwork
 
+from hardware import areca
+from hardware.benchmark import cpu as bm_cpu
+from hardware.benchmark import disk as bm_disk
+from hardware.benchmark import mem as bm_mem
+from hardware import bios_hp
 from hardware import detect_utils
 from hardware.detect_utils import cmd
 from hardware import diskinfo
@@ -40,6 +47,8 @@ from hardware import hpacucli
 from hardware import infiniband as ib
 from hardware import ipmi
 from hardware import megacli
+from hardware import rtc
+from hardware import smart_utils
 
 SIOCGIFNETMASK = 0x891b
 
@@ -49,14 +58,15 @@ def size_in_gb(size):
     ret = size.replace(' ', '')
     if ret[-2:] == 'GB':
         return ret[:-2]
-    elif ret[-2:] == 'TB':
+
+    if ret[-2:] == 'TB':
         # some size are provided in x.yGB
         # we need to compute the size in TB by
         # considering the input as a float to be
         # multiplied by 1000
         return str(int(float(ret[:-2]) * 1000))
-    else:
-        return ret
+
+    return ret
 
 
 def detect_hpa(hw_lst):
@@ -67,7 +77,7 @@ def detect_hpa(hw_lst):
         if not cli.launch():
             return False
         controllers = cli.ctrl_all_show()
-        if len(controllers) == 0:
+        if not controllers:
             sys.stderr.write("Info: No hpa controller found\n")
             return False
 
@@ -96,8 +106,8 @@ def detect_hpa(hw_lst):
                         value = disk_infos[disk_info]
                         if disk_info == "size":
                             value = size_in_gb(disk_infos[disk_info])
-                            global_pdisk_size = (global_pdisk_size +
-                                                 float(value))
+                            global_pdisk_size = (
+                                global_pdisk_size + float(value))
                         hw_lst.append(('disk', disk[0], disk_info,
                                        value))
         except hpacucli.Error as expt:
@@ -111,6 +121,51 @@ def detect_hpa(hw_lst):
     return True
 
 
+def detect_areca(hw_lst):
+    'Detect Areca controller configuration'
+    device = areca.sys_info()
+    if not device:
+        sys.stderr.write('Info: detect_areca: No controller found\n')
+        return
+    if "ControllerName" not in device.keys():
+        sys.stderr.write('Info: detect_areca: Cannot found controller name\n')
+        return
+
+    sys.stderr.write('Info: detect_areca: Found %s version %s\n' %
+                     (device['ControllerName'], device['FirmwareVersion']))
+
+    areca.disable_password()
+
+    for info in device.keys():
+        hw_lst.append(('areca', 'system', info, device[info]))
+
+    adsys = areca.adsys_info()
+    for info in adsys.keys():
+        hw_lst.append(('areca', 'system', info, adsys[info]))
+
+    cfg = areca.sys_showcfg()
+    for info in cfg.keys():
+        hw_lst.append(('areca', 'config', info, cfg[info]))
+
+    hw_info = areca.hw_info()
+    for info in hw_info.keys():
+        hw_lst.append(('areca', 'hardware', info, hw_info[info]))
+
+    pwr_info = areca.hdd_pwr_info()
+    for info in pwr_info.keys():
+        hw_lst.append(('areca', 'power', info, pwr_info[info]))
+
+    for disk_number in range(1, 255):
+        disk_info = areca.disk_info(disk_number)
+        # If we don't have info about that disk, let's stop here
+        if len(disk_info) < 2:
+            break
+        # Extracting disk information
+        for info in disk_info:
+            hw_lst.append(('areca', "disk%d" % disk_number, info,
+                           disk_info[info]))
+
+
 def detect_megacli(hw_lst):
     'Detect LSI MegaRAID controller configuration.'
     ctrl_num = megacli.adp_count()
@@ -134,7 +189,6 @@ def detect_megacli(hw_lst):
                                        '%s' % key, '%s' % enc[key]))
 
                 for slot_num in range(enc['NumberOfSlots']):
-                    disk_count += 1
                     disk = 'disk%d' % slot_num
                     info = megacli.pdinfo(ctrl,
                                           enc['DeviceId'],
@@ -144,6 +198,7 @@ def detect_megacli(hw_lst):
                     if 'PdType' not in info.keys():
                         continue
 
+                    disk_count += 1
                     hw_lst.append(('pdisk',
                                    disk,
                                    'ctrl',
@@ -172,9 +227,9 @@ def detect_megacli(hw_lst):
                         if key not in ignore_list:
                             if "DriveTemperature" in key:
                                 if "C" in str(info[key].split()[0]):
-                                    a = str(info[key].split()[0].split("C")[0])
+                                    pdisk = info[key].split()[0].split("C")[0]
                                     hw_lst.append(('pdisk', disk, key,
-                                                   a.strip()))
+                                                   str(pdisk).strip()))
                                     hw_lst.append(('pdisk', disk,
                                                    "%s_units" % key,
                                                    "Celsius"))
@@ -203,56 +258,19 @@ def detect_megacli(hw_lst):
                 for ld_num in range(megacli.ld_get_num(ctrl)):
                     disk = 'disk%d' % ld_num
                     info = megacli.ld_get_info(ctrl, ld_num)
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'current_access_policy',
-                                   info['CurrentAccessPolicy']))
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'current_cache_policy',
-                                   info['CurrentCachePolicy']))
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'disk_cache_policy',
-                                   info['DiskCachePolicy']))
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'name',
-                                   info['Name']))
-                    try:
+                    ignore_list = ['Size']
+
+                    for item in info.keys():
+                        if item not in ignore_list:
+                            hw_lst.append(('ldisk',
+                                           disk,
+                                           item,
+                                           str(info[item])))
+                    if 'Size' in info:
                         hw_lst.append(('ldisk',
                                        disk,
-                                       'number_of_drives',
-                                       str(info['NumberOfDrives'])))
-                    except KeyError:
-                        pass
-                    try:
-                        hw_lst.append(('ldisk',
-                                       disk,
-                                       'number_of_drives_per_span',
-                                       str(info['NumberOfDrivesPerSpan'])))
-                    except KeyError:
-                        pass
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'raid_level',
-                                   info['RaidLevel']))
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'sector_size',
-                                   str(info['SectorSize'])))
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'state',
-                                   info['State']))
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'Size',
-                                   size_in_gb(info['Size'])))
-                    hw_lst.append(('ldisk',
-                                   disk,
-                                   'strip_size',
-                                   info['StripSize']))
+                                       'Size',
+                                       size_in_gb(info['Size'])))
         hw_lst.append(('disk', 'megaraid', 'count', str(disk_count)))
         return True
     else:
@@ -260,52 +278,24 @@ def detect_megacli(hw_lst):
 
 
 def detect_disks(hw_lst):
-    'Detect disks.'
+    """Detect disks."""
+
     names = diskinfo.disknames()
     sizes = diskinfo.disksizes(names)
     disks = [name for name, size in sizes.items() if size > 0]
     hw_lst.append(('disk', 'logical', 'count', str(len(disks))))
     for name in disks:
-        hw_lst.append(('disk', name, 'size', str(sizes[name])))
-        item_list = ['device/vendor', 'device/model', 'device/rev',
-                     'queue/optimal_io_size', 'queue/physical_block_size',
-                     'queue/rotational']
-        for my_item in item_list:
-            try:
-                with open('/sys/block/%s/%s' % (name, my_item), 'r') as dev:
-                    hw_lst.append(('disk', name, my_item.split('/')[1],
-                                   dev.readline().rstrip('\n').strip()))
-            except Exception as excpt:
-                sys.stderr.write(
-                    'Failed at getting disk information '
-                    'at /sys/block/%s/device/%s: %s\n' % (name,
-                                                          my_item,
-                                                          str(excpt)))
+        diskinfo.get_disk_info(name, sizes, hw_lst)
 
-        item_list = ['WCE', 'RCD']
-        item_def = {'WCE': 'Write Cache Enable', 'RCD': 'Read Cache Disable'}
-        for my_item in item_list:
-            sdparm_cmd = Popen("sdparm -q --get=%s /dev/%s | "
-                               "awk '{print $2}'" % (my_item, name),
-                               shell=True,
-                               stdout=PIPE)
-            for line in sdparm_cmd.stdout:
-                hw_lst.append(('disk', name, item_def.get(my_item),
-                               line.rstrip('\n').strip()))
+        # nvme devices do not need standard cache mechanisms
+        if not name.startswith('nvme'):
+            diskinfo.get_disk_cache(name, hw_lst)
 
-        # In some VMs, the disk-by id doesn't exists
-        if os.path.exists('/dev/disk/by-id/'):
-            for entry in os.listdir('/dev/disk/by-id/'):
-                idp = os.path.realpath('/dev/disk/by-id/' + entry).split('/')
-                if idp[-1] == name:
-                    id_name = "id"
-                    if entry.startswith('wwn'):
-                        id_name = "wwn-id"
-                    elif entry.startswith('scsi'):
-                        id_name = "scsi-id"
-                    hw_lst.append(('disk', name, id_name, entry))
+        diskinfo.get_disk_id(name, hw_lst)
 
-        detect_utils.read_SMART(hw_lst, "/dev/%s" % name)
+        # TODO(rpittau): add smart-tools support for nvme devices
+        if not name.startswith('nvme'):
+            smart_utils.read_smart(hw_lst, "/dev/%s" % name)
 
 
 def modprobe(module):
@@ -321,7 +311,7 @@ def detect_ipmi(hw_lst):
     modprobe("ipmi_si")
     modprobe("ipmi_devintf")
     if (os.path.exists('/dev/ipmi0') or os.path.exists('/dev/ipmi/0') or
-       os.path.exists('/dev/ipmidev/0')):
+            os.path.exists('/dev/ipmidev/0')):
         for channel in range(0, 16):
             status, _ = cmd('ipmitool channel info %d 2>&1 | grep -sq Volatile'
                             % channel)
@@ -331,17 +321,19 @@ def detect_ipmi(hw_lst):
         status, output = cmd('ipmitool lan print')
         if status == 0:
             ipmi.parse_lan_info(output, hw_lst)
-    else:
-        # do we need a fake ipmi device for testing purpose ?
-        status, _ = cmd('grep -qi FAKEIPMI /proc/cmdline')
-        if status == 0:
-            # Yes ! So let's create a fake entry
-            hw_lst.append(('system', 'ipmi-fake', 'channel', '0'))
-            sys.stderr.write('Info: Added fake IPMI device\n')
-            return True
-        else:
-            sys.stderr.write('Info: No IPMI device found\n')
-            return False
+
+        return True
+
+    # do we need a fake ipmi device for testing purpose ?
+    status, _ = cmd('grep -qi FAKEIPMI /proc/cmdline')
+    if status == 0:
+        # Yes ! So let's create a fake entry
+        hw_lst.append(('system', 'ipmi-fake', 'channel', '0'))
+        sys.stderr.write('Info: Added fake IPMI device\n')
+        return True
+
+    sys.stderr.write('Info: No IPMI device found\n')
+    return False
 
 
 def get_cidr(netmask):
@@ -360,63 +352,97 @@ This pci device shall be from vendor Mellanox (15b3) form class 0280
 Class 280 stands for a Network Controller while ethernet device are 0200.
 '''
     status, _ = cmd("lspci -d 15b3: -n|awk '{print $2}'|grep -q '0280'")
-    if status == 0:
-        ib_card = 0
-        for devices in range(ib_card, len(ib.ib_card_drv())):
-            card_type = ib.ib_card_drv()[devices]
-            ib_infos = ib.ib_global_info(card_type)
-            nb_ports = ib_infos['nb_ports']
-            hw_lst.append(('infiniband', 'card%i' % ib_card,
-                           'card_type', card_type))
-            hw_lst.append(('infiniband', 'card%i' % ib_card,
-                           'device_type', ib_infos['device_type']))
-            hw_lst.append(('infiniband', 'card%i' % ib_card,
-                           'fw_version', ib_infos['fw_ver']))
-            hw_lst.append(('infiniband', 'card%i' % ib_card,
-                           'hw_version', ib_infos['hw_ver']))
-            hw_lst.append(('infiniband', 'card%i' % ib_card,
-                           'nb_ports', nb_ports))
-            hw_lst.append(('infiniband', 'card%i' % ib_card,
-                           'sys_guid', ib_infos['sys_guid']))
-            hw_lst.append(('infiniband', 'card%i' % ib_card,
-                           'node_guid', ib_infos['node_guid']))
-            for port in range(1, int(nb_ports) + 1):
-                ib_port_infos = ib.ib_port_info(card_type, port)
-                hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
-                               'state', ib_port_infos['state']))
-                hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
-                               'physical_state',
-                               ib_port_infos['physical_state']))
-                hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
-                               'rate', ib_port_infos['rate']))
-                hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
-                               'base_lid', ib_port_infos['base_lid']))
-                hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
-                               'lmc', ib_port_infos['lmc']))
-                hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
-                               'sm_lid', ib_port_infos['sm_lid']))
-                hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
-                               'port_guid', ib_port_infos['port_guid']))
-        return True
-    else:
+    if status != 0:
         sys.stderr.write('Info: No Infiniband device found\n')
         return False
 
+    for ib_card in range(len(ib.ib_card_drv())):
+        card_type = ib.ib_card_drv()[ib_card]
+        ib_infos = ib.ib_global_info(card_type)
+        nb_ports = ib_infos['nb_ports']
+        hw_lst.append(('infiniband', 'card%i' % ib_card,
+                       'card_type', card_type))
+        hw_lst.append(('infiniband', 'card%i' % ib_card,
+                       'device_type', ib_infos['device_type']))
+        hw_lst.append(('infiniband', 'card%i' % ib_card,
+                       'fw_version', ib_infos['fw_ver']))
+        hw_lst.append(('infiniband', 'card%i' % ib_card,
+                       'hw_version', ib_infos['hw_ver']))
+        hw_lst.append(('infiniband', 'card%i' % ib_card,
+                       'nb_ports', nb_ports))
+        hw_lst.append(('infiniband', 'card%i' % ib_card,
+                       'sys_guid', ib_infos['sys_guid']))
+        hw_lst.append(('infiniband', 'card%i' % ib_card,
+                       'node_guid', ib_infos['node_guid']))
+        for port in range(1, int(nb_ports) + 1):
+            ib_port_infos = ib.ib_port_info(card_type, port)
+            hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
+                           'state', ib_port_infos['state']))
+            hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
+                           'physical_state',
+                           ib_port_infos['physical_state']))
+            hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
+                           'rate', ib_port_infos['rate']))
+            hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
+                           'base_lid', ib_port_infos['base_lid']))
+            hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
+                           'lmc', ib_port_infos['lmc']))
+            hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
+                           'sm_lid', ib_port_infos['sm_lid']))
+            hw_lst.append(('infiniband', 'card%i_port%i' % (ib_card, port),
+                           'port_guid', ib_port_infos['port_guid']))
+    return True
 
-def get_uuid():
+
+def _get_uuid_x86_64():
     'Get uuid from dmidecode'
     uuid_cmd = Popen("dmidecode -t 1 | grep UUID | "
                      "awk '{print $2}'",
                      shell=True,
-                     stdout=PIPE)
-    return uuid_cmd.stdout.read().rstrip()
+                     stdout=PIPE,
+                     universal_newlines=True)
+    stdout = uuid_cmd.communicate()[0]
+    return stdout.rstrip()
+
+
+def _get_uuid_ppc64le(hw_lst):
+    vendor = None
+    serial = None
+    for (_, _, sys_subtype, value) in hw_lst:
+        if sys_subtype == 'vendor':
+            vendor = value
+        if sys_subtype == 'serial':
+            serial = value
+
+    system_uuid = None
+    system_uuid_fname = '/sys/firmware/devicetree/base/system-uuid'
+    if os.access(system_uuid_fname, os.R_OK):
+        with open(system_uuid_fname) as uuidfile:
+            system_uuid = uuidfile.read().rstrip(' \t\r\n\0')
+    elif vendor and serial:
+        root = uuid.UUID(bytes=b'\x00' * 16)
+        vendor_uuid = uuid.uuid5(root, vendor)
+        system_uuid = str(uuid.uuid5(vendor_uuid, serial))
+
+    return system_uuid
+
+
+def get_uuid(hw_lst):
+    if os.uname()[4] == 'ppc64le':
+        return _get_uuid_ppc64le(hw_lst)
+    return _get_uuid_x86_64()
+
+
+def _get_value(hw_lst, *vect):
+    for i in hw_lst:
+        if i[0:3] == vect:
+            return i[3]
+    return ''
 
 
 def detect_system(hw_lst, output=None):
     'Detect system characteristics from the output of lshw.'
 
-    socket_count = 0
-
     def find_element(xml, xml_spec, sys_subtype,
                      sys_type='product', sys_cls='system',
                      attrib=None, transform=None):
@@ -439,26 +465,23 @@ def detect_system(hw_lst, output=None):
     else:
         status, output = cmd('lshw -xml')
     if status == 0:
+        mobo_id = ''
+        nic_id = ''
         xml = ET.fromstring(output)
         find_element(xml, "./node/serial", 'serial')
         find_element(xml, "./node/product", 'name')
         find_element(xml, "./node/vendor", 'vendor')
         find_element(xml, "./node/version", 'version')
-        uuid = get_uuid()
+        uuid = get_uuid(hw_lst)
+
         if uuid:
-            # If we have an uuid, let's manage a quirk list of stupid
-            # serial numbers TYAN or Supermicro are known to provide
-            # dirty serial numbers In that case, let's use the uuid
-            # instead
-            for i in hw_lst:
-                if 'system' in i[0] and 'product' in i[1] and 'serial' in i[2]:
-                    # Does the current serial number is part of the quirk list
-                    if i[3] in ['0123456789']:
-                        # Let's delete the stupid SN and use the UUID instead
-                        hw_lst.remove(i)
-                        hw_lst.append(('system', 'product', 'serial', uuid))
-                        break
-            hw_lst.append(('system', 'product', 'uuid', uuid))
+            # If we have an uuid, we shall check if it's part of a
+            # known list of broken uuid
+            # If so let's delete the uuid instead of reporting a stupid thing
+            if uuid in ['Not']:
+                uuid = ''
+            else:
+                hw_lst.append(('system', 'product', 'uuid', uuid))
 
         for elt in xml.findall(".//node[@id='core']"):
             name = elt.find('physid')
@@ -468,6 +491,7 @@ def detect_system(hw_lst, output=None):
                 find_element(elt, 'version', 'version', 'motherboard',
                              'system')
                 find_element(elt, 'serial', 'serial', 'motherboard', 'system')
+                mobo_id = _get_value(hw_lst, 'system', 'motherboard', 'serial')
 
         for elt in xml.findall(".//node[@id='firmware']"):
             name = elt.find('physid')
@@ -573,51 +597,21 @@ def detect_system(hw_lst, output=None):
                     find_element(elt, 'serial', 'serial', name.text, 'network',
                                  transform=lambda x: x.lower())
 
+                if not nic_id:
+                    nic_id = _get_value(hw_lst, 'network',
+                                        name.text, 'serial')
+                    nic_id = nic_id.replace(':', '')
+
                 detect_utils.get_ethtool_status(hw_lst, name.text)
                 detect_utils.get_lld_status(hw_lst, name.text)
 
-        for elt in xml.findall(".//node[@class='processor']"):
-            name = elt.find('physid')
-            if name is not None:
-                hw_lst.append(('cpu', 'physical_%s' % (socket_count),
-                               'physid', name.text))
-                find_element(elt, 'product', 'product',
-                             'physical_%s' % socket_count, 'cpu')
-                find_element(elt, 'vendor', 'vendor',
-                             'physical_%s' % socket_count, 'cpu')
-                find_element(elt, 'version', 'version',
-                             'physical_%s' % socket_count, 'cpu')
-                find_element(elt, 'size', 'frequency',
-                             'physical_%s' % socket_count, 'cpu')
-                find_element(elt, 'clock', 'clock',
-                             'physical_%s' % socket_count, 'cpu')
-                find_element(elt, "configuration/setting[@id='cores']",
-                             'cores', 'physical_%s' % socket_count,
-                             'cpu', 'value')
-                find_element(elt, "configuration/setting[@id='enabledcores']",
-                             'enabled_cores', 'physical_%s' % socket_count,
-                             'cpu', 'value')
-                find_element(elt, "configuration/setting[@id='threads']",
-                             'threads', 'physical_%s' % socket_count, 'cpu',
-                             'value')
+        fix_bad_serial(hw_lst, uuid, mobo_id, nic_id)
 
-                elt_cap = elt.findall("capabilities/capability")
-                cpu_flags = ""
-                for cap in elt_cap:
-                    for element in cap.items():
-                        cpu_flags = "%s %s" % (cpu_flags, element[1].strip())
-
-                hw_lst.append(('cpu', 'physical_%s' % (socket_count),
-                               'flags', cpu_flags.strip()))
-
-                socket_count = socket_count + 1
     else:
         sys.stderr.write("Unable to run lshw: %s\n" % output)
+        return False
 
-    hw_lst.append(('cpu', 'physical', 'number', str(socket_count)))
-    status, output = detect_utils.cmd('nproc')
-    if status == 0:
-        hw_lst.append(('cpu', 'logical', 'number', str(output).strip()))
+    get_cpus(hw_lst)
 
     osvendor_cmd = detect_utils.output_lines("lsb_release -is")
     for line in osvendor_cmd:
@@ -640,9 +634,166 @@ def detect_system(hw_lst, output=None):
     for line in cmdline_cmd:
         hw_lst.append(('system', 'kernel', 'cmdline',
                        line.rstrip('\n').strip()))
+    return True
 
 
-def read_hwmon(hw, entry, sensor, label_name, appendix, processor_num,
+def _from_file(fname, mapping=None, default=None):
+    file_exists = os.path.exists(fname)
+    value = None
+    if file_exists:
+        with open(fname) as f:
+            value = f.readline().rstrip('\n')
+        if mapping:
+            value = mapping.get(value, default)
+    return (file_exists, value)
+
+
+def get_cpus(hw_lst):
+    def _maybe_int(v):
+        try:
+            base = 10
+            if 'x' in v:
+                base = 16
+            v = int(v, base)
+        except Exception:
+            pass
+        return v
+
+    # Extracting lspcu information
+    lscpu = {}
+    output = detect_utils.output_lines('LANG=en_US.UTF-8 lscpu')
+
+    for line in output:
+        if ':' in line:
+            item, value = line.split(':')
+            lscpu[item.strip(':')] = value.strip()
+
+    # Extracting lspcu -x information
+    lscpux = {}
+    output = detect_utils.output_lines('LANG=en_US.UTF-8 lscpu -x')
+
+    for line in output:
+        if ':' in line:
+            item, value = line.split(':')
+            lscpux[item.strip(':')] = value.strip()
+
+    hw_lst.append(("cpu", "physical", "number", int(lscpu["Socket(s)"])))
+    for processor in range(int(lscpu["Socket(s)"])):
+        ptag = "physical_{}".format(processor)
+        (exists, value) = _from_file("/sys/devices/system/cpu/cpufreq/boost",
+                                     mapping={'1': 'enabled'},
+                                     default='disabled')
+        if exists:
+            hw_lst.append(('cpu', ptag, 'boost', value))
+
+        for (t_key, d_key, conv) in [('vendor', 'Vendor ID', None),
+                                     ('product', 'Model name', None),
+                                     ('cores', 'Core(s) per socket', int),
+                                     ('threads', None, None),
+                                     ('family', 'CPU family', int),
+                                     ('model', 'Model', _maybe_int),
+                                     ('stepping', 'Stepping', _maybe_int),
+                                     ('l1d cache', 'L1d cache', None),
+                                     ('l1i cache', 'L1i cache', None),
+                                     ('l2 cache', 'L2 cache', None),
+                                     ('l3 cache', 'L3 cache', None),
+                                     ('min_Mhz', 'CPU min MHz', float),
+                                     ('max_Mhz', 'CPU max MHz', float),
+                                     ('current_Mhz', 'CPU MHz', float),
+                                     ('flags', 'Flags', None)]:
+            value = None
+            if d_key in lscpu:
+                value = lscpu[d_key]
+                if conv:
+                    value = conv(value)
+            elif t_key == 'threads':
+                value = (int(lscpu.get('Thread(s) per core', 1)) *
+                         int(lscpu.get('Core(s) per socket', 1)))
+            if value is not None:
+                hw_lst.append(('cpu', ptag, t_key, value))
+
+    hw_lst.append(('cpu', 'logical', 'number', int(lscpu['CPU(s)'])))
+    # Governors could be different on logical cpus
+    for cpu in range(int(lscpu['CPU(s)'])):
+        ltag = "logical_{}".format(cpu)
+        (exists, value) = _from_file(("/sys/devices/system/cpu/cpufreq/"
+                                      "policy{}/scaling_governor".format(cpu)))
+        if exists:
+            hw_lst.append(('cpu', ltag, "governor", value))
+
+    # Extracting numa nodes
+    hw_lst.append(('numa', 'nodes', 'count', int(lscpu['NUMA node(s)'])))
+
+    # Allow for sparse numa nodes.
+    numa_nodes = []
+    for key in lscpux:
+        match = re.match('NUMA node(\d+) CPU\(s\)', key)
+        if match:
+            numa_nodes.append((key, int(match.groups()[0])))
+    # NOTE(tonyb): Explictly sort the list as prior to python 3.7? keys() did
+    # not have a predictable ordering and there maybe consumers of hw_lst rely
+    # on that.
+    numa_nodes.sort(key=lambda t: t[1])
+    for (key, node_id) in numa_nodes:
+        ntag = 'node_{}'.format(node_id)
+        cpus = lscpu[key]
+        # lscpu -x provides the cpu mask
+        cpu_mask = lscpux[key]
+        total_cpus = 0
+        min_cpu = None
+        max_cpu = None
+
+        # It's possible to have a NUMA node without any CPUs
+        if cpus:
+            for item in cpus.split(','):
+                # lscpu report numa nodes like 0-5,48-53
+                if "-" in item:
+                    max_cpu = int(item.split("-")[1])
+                    min_cpu = int(item.split("-")[0])
+                    total_cpus = total_cpus + max_cpu - min_cpu + 1
+                else:
+                    # or like 0,1
+                    if min_cpu is None:
+                        min_cpu = int(item)
+                    else:
+                        max_cpu = int(item)
+                        total_cpus = total_cpus + max_cpu - min_cpu + 1
+
+        # total_cpus = 12 for "0-5,48-53"
+        hw_lst.append(('numa', ntag, 'cpu_count', total_cpus))
+        hw_lst.append(('numa', ntag, 'cpu_mask', cpu_mask))
+
+
+def fix_bad_serial(hw_lst, uuid, mobo_id, nic_id):
+    'Fix bad serial number'
+    # Let's manage a quirk list of stupid serial numbers TYAN
+    # or Supermicro are known to provide dirty serial numbers
+    # In that case, let's use another serial
+    for i in hw_lst:
+        if i[0:3] == ('system', 'product', 'serial'):
+            # Does the current serial number is part of the quirk list
+            if i[3] in ['0123456789', '0000000000']:
+
+                # Let's delete the stupid SN and use the another ID instead
+                # Items are ordered by level of confidence
+                new_serial = ''
+
+                if uuid:
+                    new_serial = uuid
+                elif mobo_id:
+                    new_serial = mobo_id
+                elif nic_id:
+                    new_serial = nic_id
+
+                if new_serial:
+                    hw_lst.remove(i)
+                    hw_lst.append(('system', 'product', 'serial',
+                                   new_serial))
+
+                break
+
+
+def read_hwmon(hwlst, entry, sensor, label_name, appendix, processor_num,
                entry_name):
     try:
         hwmon = "%s_%s" % (sensor, appendix)
@@ -662,13 +813,13 @@ def read_hwmon(hw, entry, sensor, label_name, appendix, processor_num,
                 return
 
         value = open(filename, 'r').readline().strip()
-        hw.append(('cpu', 'physical_%d' % processor_num, "%s/%s" %
-                   (label_name, entry_name), value))
+        hwlst.append(('cpu', 'physical_%d' % processor_num, "%s/%s" %
+                      (label_name, entry_name), value))
     except Exception:
         pass
 
 
-def detect_temperatures(hw):
+def detect_temperatures(hwlst):
     for entry in os.listdir("/sys/devices/platform/"):
         if entry.startswith("coretemp."):
             processor_num = int(entry.split(".")[1])
@@ -677,24 +828,29 @@ def detect_temperatures(hw):
                     sensor = label.split("_")[0]
                     try:
                         with open("/sys/devices/platform/%s/%s_label" %
-                                  (entry, sensor), 'r') as f:
-                            label_name = f.readline().strip().replace(" ", "_")
+                                  (entry, sensor), 'r') as fsensor:
+                            label_name = fsensor.readline()
+                            label_name = label_name.strip().replace(" ", "_")
                     except Exception:
                         sys.stderr.write("detect_temperatures: "
                                          "Cannot open label on %s/%s\n" %
                                          (entry, sensor))
                         continue
 
-                    read_hwmon(hw, entry, sensor, label_name, "input",
+                    read_hwmon(hwlst, entry, sensor, label_name, "input",
                                processor_num, "temperature")
-                    read_hwmon(hw, entry, sensor, label_name, "max",
+                    read_hwmon(hwlst, entry, sensor, label_name, "max",
                                processor_num, "max")
-                    read_hwmon(hw, entry, sensor, label_name, "crit",
+                    read_hwmon(hwlst, entry, sensor, label_name, "crit",
                                processor_num, "critical")
-                    read_hwmon(hw, entry, sensor, label_name, "crit_alarm",
+                    read_hwmon(hwlst, entry, sensor, label_name, "crit_alarm",
                                processor_num, "critical_alarm")
 
 
+def detect_rtc_clock(hw_lst):
+    hw_lst.append(('system', 'rtc', 'utc', rtc.get_rtc()))
+
+
 def parse_ahci(hrdw, words):
     if len(words) < 4:
         return
@@ -719,50 +875,77 @@ def parse_dmesg(hrdw, output):
             parse_ahci(hrdw, words)
 
 
-def _main(options):
-    'Command line entry point.'
+def clean_str(val):
+    'Cleanup a bad string (invalid UTF-8 encoding)'
+    if isinstance(val, bytes):
+        val = val.decode('UTF-8', 'replace')
+    return val
+
+
+def clean_tuples(lst):
+    'Clean a list of tuples from bad UTF-8 strings'
+    return [tuple([clean_str(val) for val in elt]) for elt in lst]
+
+
+def parse_args(arguments):
+    """Arguments parser."""
+
+    parser = argparse.ArgumentParser()
+    parser.add_argument('-H', '--human',
+                        help='Print output in human readable format',
+                        action='store_true',
+                        default=False)
+
+    benchmark = parser.add_argument_group('benchmark')
+    benchmark.add_argument('--benchmark', '-b',
+                           choices=['cpu', 'mem', 'disk'],
+                           nargs='+',
+                           help=('Run benchmark for specific components. '
+                                 'Valid components are: cpu, mem, disk'))
+    benchmark.add_argument('--benchmark-disk-destructive',
+                           help=('If specified make the disk component '
+                                 'benchmark to be destructive'),
+                           action='store_true',
+                           default=False)
+
+    return parser.parse_args(arguments)
+
+
+def main():
+    """Command line entry point."""
+
+    os.environ["LANG"] = "en_US.UTF-8"
+    args = parse_args(sys.argv[1:])
+
     hrdw = []
+    detect_areca(hrdw)
     detect_hpa(hrdw)
     detect_megacli(hrdw)
     detect_disks(hrdw)
-    detect_system(hrdw)
+    if not detect_system(hrdw):
+        sys.exit(1)
     detect_ipmi(hrdw)
     detect_infiniband(hrdw)
     detect_temperatures(hrdw)
     detect_utils.get_ddr_timing(hrdw)
     detect_utils.ipmi_sdr(hrdw)
-    status, output = cmd("dmesg")
+    detect_rtc_clock(hrdw)
+    _, output = cmd("dmesg")
     parse_dmesg(hrdw, output)
-    if "human" in options.keys():
+    bios_hp.dump_hp_bios(hrdw)
+
+    if args.benchmark:
+        if 'cpu' in args.benchmark:
+            bm_cpu.cpu_perf(hrdw)
+        if 'mem' in args.benchmark:
+            bm_mem.mem_perf(hrdw)
+        if 'disk' in args.benchmark:
+            bm_disk.disk_perf(hrdw,
+                              destructive=args.benchmark_disk_destructive)
+
+    hrdw = clean_tuples(hrdw)
+
+    if args.human:
         pprint.pprint(hrdw)
     else:
         print(json.dumps(hrdw))
-
-
-def print_help():
-    print('detect.py help page')
-    print()
-    print('-h or --help    : Print this help')
-    print('-H or --human   : Print output in human readable format')
-    print()
-
-
-def main():
-    options = {}
-
-    try:
-        opts, args = getopt.getopt(sys.argv[1:], "hH", ['help', 'human'])
-    except getopt.GetoptError:
-        print("Error: One of the options passed to the"
-              " cmdline was not supported")
-        print("Please fix your command line or read the help (-h option)")
-        sys.exit(2)
-
-    for opt, arg in opts:
-        if opt in ("-h", "--help"):
-            print_help()
-            sys.exit(0)
-        elif opt in ("-H", "--human"):
-            options["human"] = True
-
-    _main(options)
