@@ -22,6 +22,9 @@ import re
 import sys
 import types
 
+from six.moves import range
+
+
 _PREFIX = None
 
 
@@ -63,7 +66,7 @@ def _generate_values(pattern, prefix=_PREFIX):
 
 Ranges are defined like 10-12:15-18 or from a list of entries.
 '''
-    if isinstance(pattern, list) or isinstance(pattern, tuple):
+    if isinstance(pattern, list):
         for elt in pattern:
             yield elt
     elif isinstance(pattern, dict):
@@ -80,9 +83,7 @@ Ranges are defined like 10-12:15-18 or from a list of entries.
             yield pattern
     elif isinstance(pattern, str):
         parts = pattern.split('.')
-        if (_IPV4_RANGE_REGEXP.search(pattern) and
-            len(parts) == 4 and (pattern.find(':') != -1 or
-                                 pattern.find('-') != -1)):
+        if (_IPV4_RANGE_REGEXP.search(pattern) and len(parts) == 4 and (pattern.find(':') != -1 or pattern.find('-') != -1)):
             gens = [_generate_range(part) for part in parts]
             for part0 in gens[0]:
                 for part1 in gens[1]:
@@ -100,10 +101,10 @@ Ranges are defined like 10-12:15-18 or from a list of entries.
                 for num in _generate_range(res.group(2)):
                     yield head + num + foot
             else:
-                for _ in xrange(16387064):
+                for _ in range(16387064):
                     yield pattern
     else:
-        for _ in xrange(16387064):
+        for _ in range(16387064):
             yield pattern
 
 
@@ -126,8 +127,8 @@ def _call_nexts(model):
     # We can have nested generators so call again
     if generated:
         return _call_nexts(entry)
-    else:
-        return entry
+
+    return entry
 
 
 def generate(model, prefix=_PREFIX):
@@ -179,7 +180,7 @@ def generate_dict(model, prefix=_PREFIX):
             for newkey, val in zip(list(_generate_values(key, prefix)),
                                    generate(model[thekey], prefix)):
                 try:
-                    result[newkey] = merge(result[key], val)
+                    result[newkey] = merge(result[newkey], val)
                 except KeyError:
                     result[newkey] = val
         else:
