@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2013-2014 eNovance SAS <licensing@enovance.com>
+# Copyright (C) 2013-2015 eNovance SAS <licensing@enovance.com>
 #
 # Author: Frederic Lepied <frederic.lepied@enovance.com>
 #
@@ -17,16 +17,13 @@
 
 '''Functions to match according to a requirement specification.'''
 
+import ipaddress
 import logging
 import re
 import sys
-try:
-    import ipaddr
-    _HAS_IPADDR = True
-except ImportError:
-    _HAS_IPADDR = False
 
-_FUNC_REGEXP = re.compile(r'^(.*)\((.*)\)')
+
+LOG = logging.getLogger('hardware.matcher')
 
 
 def _adder(array, index, value):
@@ -42,55 +39,91 @@ def _appender(array, index, value):
         array[index] = [value, ]
 
 
+def _range(elt, minval, maxval):
+    'Helper for match_spec.'
+    return float(elt) >= float(minval) and float(elt) <= float(maxval)
+
+
 def _gt(left, right):
     'Helper for match_spec.'
-    return int(left) > int(right)
+    return float(left) > float(right)
 
 
 def _ge(left, right):
     'Helper for match_spec.'
-    return int(left) >= int(right)
+    return float(left) >= float(right)
 
 
 def _lt(left, right):
     'Helper for match_spec.'
-    return int(left) < int(right)
+    return float(left) < float(right)
 
 
 def _le(left, right):
     'Helper for match_spec.'
-    return int(left) <= int(right)
+    return float(left) <= float(right)
+
+
+def _not(_, right):
+    'Helper for match_spec.'
+    return not right
+
+
+def _and(_, left, right):
+    'Helper for match_spec.'
+    return left and right
+
+
+def _or(_, left, right):
+    'Helper for match_spec.'
+    return left or right
 
 
 def _network(left, right):
     'Helper for match_spec.'
-    if _HAS_IPADDR:
-        return ipaddr.IPv4Address(left) in ipaddr.IPv4Network(right)
-    else:
-        return False
+    return ipaddress.IPv4Address(left) in ipaddress.IPv4Network(right)
 
 
 def _regexp(left, right):
     'Helper for match_spec.'
-    print(left, right)
     return re.search(right, left) is not None
 
 
-def _in(elt, _list):
+def _in(elt, *lst):
     'Helper for match_spec.'
-    # build a list from the string or return False
-    try:
-        lst = eval('(' + _list + ')')
-    except Exception:
-        return False
-    # cast into an int or do nothing
-    try:
-        elt = int(elt)
-    except ValueError:
-        pass
     return elt in lst
 
 
+_FUNC_REGEXP = re.compile(r'^([^(]+)'          # function name
+                          r'\(\s*([^,]+)'      # first argument
+                          r'(?:\s*,\s*(.+))?'  # remaining optional arguments
+                          r'\)$')              # last parenthesis
+
+
+def _call_func(func, implicit, res):
+    'Helper function for extract_result and match_spec'
+    args = [implicit, res.group(2)]
+    # split the optional arguments if we have some
+    if res.group(3):
+        args = args + re.split(r'\s*,\s*', res.group(3))
+    # remove strings delimiters
+    args = [x.strip('\'"') for x in args]
+    # call function
+    args = [_extract_result(implicit, x) for x in args]
+    return func(*args)
+
+
+def _extract_result(implicit, expr):
+    'Helper function for match_spec'
+    res = _FUNC_REGEXP.search(expr)
+    if res:
+        func_name = '_' + res.group(1)
+        if func_name in globals():
+            return _call_func(globals()[func_name], implicit, res)
+
+    return expr
+
+
 def match_spec(spec, lines, arr, adder=_adder):
     'Match a line according to a spec and store variables in <var>.'
     # match a line without variable
@@ -120,7 +153,8 @@ def match_spec(spec, lines, arr, adder=_adder):
                 if res:
                     func_name = '_' + res.group(1)
                     if func_name in globals():
-                        if not globals()[func_name](line[idx], res.group(2)):
+                        if not _call_func(globals()[func_name],
+                                          line[idx], res):
                             if var == func:
                                 break
                         else:
@@ -165,19 +199,26 @@ Store variables starting with a $ in <arr>. Variables starting with
     if level == 50:
         return False
     # Match lines using specs
-    while len(specs) > 0:
+    while specs:
         copy_specs = list(specs)
         spec = specs.pop(0)
         line = match_spec(spec, lines, arr)
+        if debug:
+            sys.stderr.write('match_spec: %s %s\n' % (line, spec))
         # No match
         if not line:
             # Backtrack on the backtracking points
-            while len(points) > 0:
+            while points:
                 lines, specs, new_arr = points.pop()
+                if debug:
+                    sys.stderr.write('retrying with: %s\n' %
+                                     (new_arr,))
                 if match_all(lines, specs, new_arr, arr2, debug, level + 1):
                     # Copy arr back
                     for k in new_arr:
                         arr[k] = new_arr[k]
+                    if debug:
+                        sys.stderr.write('success: %d\n' % level)
                     return True
             if level == 0 and debug:
                 sys.stderr.write('spec: %s not matched\n' % str(spec))
@@ -189,7 +230,10 @@ Store variables starting with a $ in <arr>. Variables starting with
                 # Put the matching line at the end of the lines
                 copy_lines.append(line)
                 points.append((copy_lines, copy_specs, copy_arr))
-                copy_arr = arr
+                copy_arr = dict(arr)
+                if debug:
+                    sys.stderr.write('new var: %s %s\n' % (arr, line))
+
     # Manage $$ variables
     for key in arr:
         if key[0] == '$':
@@ -224,34 +268,27 @@ As a result, we do have a filename like :
     sysvars = {}
     sysvars['sysname'] = ''
 
-    match_spec(('system', 'product', 'vendor', '$sysprodvendor'),
-               hw_items, sysvars)
-
-    if 'sysprodvendor' in sysvars:
+    if match_spec(('system', 'product', 'vendor', '$sysprodvendor'),
+                  hw_items, sysvars):
         sysvars['sysname'] += (re.sub(r'\W+', '', sysvars['sysprodvendor']) +
                                '-')
 
-    match_spec(('system', 'product', 'name', '$sysprodname'),
-               hw_items, sysvars)
-
-    if 'sysprodname' in sysvars:
+    if match_spec(('system', 'product', 'name', '$sysprodname'),
+                  hw_items, sysvars):
         sysvars['sysname'] = re.sub(r'\W+', '', sysvars['sysprodname']) + '-'
 
-    match_spec(('system', 'product', 'serial', '$sysserial'),
-               hw_items, sysvars)
-
     # Let's use any existing DMI serial number or take the first mac address
-    if 'sysserial' in sysvars:
-        sysvars['sysname'] += re.sub(r'\W+', '', sysvars['sysserial'])
+    if match_spec(('system', 'product', 'serial', '$sysserial'),
+                  hw_items, sysvars):
+        sysvars['sysname'] += re.sub(r'\W+', '', sysvars['sysserial']) + '-'
 
     # we always need to have the mac addresses for pxemngr
     if match_multiple(hw_items,
                       ('network', '$eth', 'serial', '$serial'),
                       sysvars):
-        if 'sysserial' not in sysvars:
-            sysvars['sysname'] += sysvars['serial'][0].replace(':', '-')
+        sysvars['sysname'] += sysvars['serial'][0].replace(':', '-')
     else:
-        logging.warning('unable to detect network macs')
+        LOG.warning('unable to detect network macs')
 
     return sysvars
 
