@@ -15,6 +15,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from collections import OrderedDict
 import unittest
 
 from hardware import generate
@@ -79,7 +80,7 @@ class TestGenerate(unittest.TestCase):
         self.assertEqual(
             generate.generate(model),
             [{'gw': '192.168.1.1'}]
-            )
+        )
 
     def test_generate(self):
         model = {'gw': '192.168.1.1',
@@ -90,7 +91,7 @@ class TestGenerate(unittest.TestCase):
             [{'gw': '192.168.1.1', 'ip': '192.168.1.10', 'hostname': 'host10'},
              {'gw': '192.168.1.1', 'ip': '192.168.1.11', 'hostname': 'host11'},
              {'gw': '192.168.1.1', 'ip': '192.168.1.12', 'hostname': 'host12'}]
-            )
+        )
 
     def test_generate_with_zeros(self):
         model = {'gw': '192.168.1.1',
@@ -105,7 +106,7 @@ class TestGenerate(unittest.TestCase):
              {'gw': '192.168.1.1', 'ip': '192.168.1.5', 'hostname': 'ceph005'},
              {'gw': '192.168.1.1', 'ip': '192.168.1.6', 'hostname': 'ceph006'},
              ]
-            )
+        )
 
     def test_generate_253(self):
         result = generate.generate({'hostname': '10.0.1-2.2-254'})
@@ -123,13 +124,13 @@ class TestGenerate(unittest.TestCase):
             result)
 
     def test_generate_list(self):
-        result = generate.generate({'hostname': ('hosta', 'hostb', 'hostc')})
+        result = generate.generate({'hostname': ['hosta', 'hostb', 'hostc']})
         self.assertEqual(
             result,
             [{'hostname': 'hosta'},
              {'hostname': 'hostb'},
              {'hostname': 'hostc'}]
-            )
+        )
 
     def test_generate_none(self):
         model = {'gateway': '10.66.6.1',
@@ -162,19 +163,21 @@ class TestGenerate(unittest.TestCase):
               {'gw': False,
                'ip': '192.168.1.12',
                'hostname': 'host12'}}]
-            )
+        )
 
     def test_generate_hosts(self):
-        model = {'=host10-12':
-                 {'=cmdb':
-                  {'gw': ['192.168.1.1', '192.168.1.2'],
-                   '=ip': '192.168.1.10-12'}}}
+        model = OrderedDict([('host10', {'foo': 'bar'}),
+                             ('=host10-12',
+                              {'=cmdb':
+                               {'gw': ['192.168.1.1', '192.168.1.2'],
+                                '=ip': '192.168.1.10-12'}})])
         self.assertEqual(
             generate.generate_dict(model, prefix='='),
             {'host10':
              {'cmdb':
               {'gw': ['192.168.1.1', '192.168.1.2'],
-               'ip': '192.168.1.10'}},
+               'ip': '192.168.1.10'},
+              'foo': 'bar'},
              'host11':
              {'cmdb':
               {'gw': ['192.168.1.1', '192.168.1.2'],
@@ -183,7 +186,31 @@ class TestGenerate(unittest.TestCase):
              {'cmdb':
               {'gw': ['192.168.1.1', '192.168.1.2'],
                'ip': '192.168.1.12'}}}
-            )
+        )
+
+    def test_generate_tuple(self):
+        disk_struct = (
+            {'size_gb': 50,
+             'raid_level': '1+0',
+             'disk_type': 'hdd',
+             'interface_type': 'sas',
+             'volume_name': 'root_volume',
+             'is_root_volume': 'true'},
+            {'size_gb': 100,
+             'number_of_physical_disks': 3,
+             'raid_level': '5',
+             'disk_type': 'hdd',
+             'interface_type': 'sas',
+             'volume_name': 'data_volume'}
+        )
+        model = {
+            'hostname': 'node1-2',
+            'logical_disks': disk_struct
+        }
+        result = generate.generate(model)
+        self.assertEqual(len(result), 2)
+        self.assertEqual(result[0]['logical_disks'], disk_struct)
+        self.assertEqual(result[1]['logical_disks'], disk_struct)
 
 
 class TestMerge(unittest.TestCase):
@@ -212,6 +239,7 @@ class TestMerge(unittest.TestCase):
         generate.merge(dic1, dic2)
         self.assertEqual(dic1['a'], [1, 2, 3, 4])
 
+
 if __name__ == "__main__":
     unittest.main()
 
