@@ -21,6 +21,4 @@ _BASEDIR = os.path.dirname(os.path.abspath(__file__))
 
 
 def sample(name):
-    return open(os.path.join(_BASEDIR, "samples", name)).read()
-
-# utils.py ends here
+    return open(os.path.join(_BASEDIR, 'samples', name)).read()
